@@ -1,6 +1,9 @@
+from __future__ import division, print_function, absolute_import
+
 import numpy as np
 from scipy.cluster import vq
 
+
 def python_vq(all_data,code_book):
     import time
     t1 = time.time()
@@ -24,13 +27,15 @@ def python_vq(all_data,code_book):
 
     return codes1,dist1
 
+
 def read_data(name):
     f = open(name,'r')
     data = []
     for line in f.readlines():
-        data.append(map(float,string.split(line)))
+        data.append(list(map(float, line.split())))
     f.close()
-    return array(data)
+    return np.array(data)
+
 
 def main():
     np.random.seed((1000,1000))
