@@ -18,6 +18,7 @@ from numpy.testing import assert_, assert_equal, assert_array_almost_equal, \
         TestCase, dec
 from scipy.fftpack import ifft,fft,fftn,ifftn,rfft,irfft, fft2
 from scipy.fftpack import _fftpack as fftpack
+from scipy.fftpack.basic import _is_safe_size
 
 from numpy import arange, add, array, asarray, zeros, dot, exp, pi,\
      swapaxes, double, cdouble
@@ -44,14 +45,18 @@ SMALL_PRIME_SIZES = [
 ]
 
 from numpy.random import rand
+
+
 def random(size):
     return rand(*size)
 
+
 def get_mat(n):
     data = arange(n)
     data = add.outer(data,data)
     return data
 
+
 def direct_dft(x):
     x = asarray(x)
     n = len(x)
@@ -61,6 +66,7 @@ def direct_dft(x):
         y[i] = dot(exp(i*w),x)
     return y
 
+
 def direct_idft(x):
     x = asarray(x)
     n = len(x)
@@ -70,18 +76,21 @@ def direct_idft(x):
         y[i] = dot(exp(i*w),x)/n
     return y
 
+
 def direct_dftn(x):
     x = asarray(x)
     for axis in range(len(x.shape)):
         x = fft(x,axis=axis)
     return x
 
+
 def direct_idftn(x):
     x = asarray(x)
     for axis in range(len(x.shape)):
         x = ifft(x,axis=axis)
     return x
 
+
 def direct_rdft(x):
     x = asarray(x)
     n = len(x)
@@ -91,27 +100,29 @@ def direct_rdft(x):
         y = dot(exp(i*w),x)
         if i:
             r[2*i-1] = y.real
-            if 2*i<n:
+            if 2*i < n:
                 r[2*i] = y.imag
         else:
             r[0] = y.real
     return r
 
+
 def direct_irdft(x):
     x = asarray(x)
     n = len(x)
     x1 = zeros(n,dtype=cdouble)
     for i in range(n//2+1):
         if i:
-            if 2*i<n:
-                x1[i] = x[2*i-1] + 1j* x[2*i]
-                x1[n-i] = x[2*i-1] - 1j* x[2*i]
+            if 2*i < n:
+                x1[i] = x[2*i-1] + 1j*x[2*i]
+                x1[n-i] = x[2*i-1] - 1j*x[2*i]
             else:
                 x1[i] = x[2*i-1]
         else:
             x1[0] = x[0]
     return direct_idft(x1).real
 
+
 class _TestFFTBase(TestCase):
     def setUp(self):
         self.cdt = None
@@ -119,13 +130,13 @@ class _TestFFTBase(TestCase):
         np.random.seed(1234)
 
     def test_definition(self):
-        x = np.array([1,2,3,4+1j,1,2,3,4+2j], dtype = self.cdt)
+        x = np.array([1,2,3,4+1j,1,2,3,4+2j], dtype=self.cdt)
         y = fft(x)
         self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         y1 = direct_dft(x)
         assert_array_almost_equal(y,y1)
-        x = np.array([1,2,3,4+0j,5], dtype = self.cdt)
+        x = np.array([1,2,3,4+0j,5], dtype=self.cdt)
         assert_array_almost_equal(fft(x),direct_dft(x))
 
     def test_n_argument_real(self):
@@ -158,11 +169,24 @@ class _TestFFTBase(TestCase):
             y = fftpack.zrfft(x)
             assert_array_almost_equal(y,y2)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, fft, [])
+        assert_raises(ValueError, fft, [[1,1],[2,2]], -5)
+
+    def test__is_safe_size(self):
+        vals = [(0, True), (1, True), (2, True), (3, True), (4, True), (5, True), (6, True), (7, False),
+                (15, True), (16, True), (17, False), (18, True), (21, False), (25, True), (50, True),
+                (120, True), (210, False)]
+        for n, is_safe in vals:
+            assert_equal(_is_safe_size(n), is_safe)
+
+
 class TestDoubleFFT(_TestFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
 
+
 class TestSingleFFT(_TestFFTBase):
     def setUp(self):
         self.cdt = np.complex64
@@ -172,6 +196,7 @@ class TestSingleFFT(_TestFFTBase):
     def test_notice(self):
         pass
 
+
 class _TestIFFTBase(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -213,15 +238,15 @@ class _TestIFFTBase(TestCase):
     def test_random_complex(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
             x = random([size]).astype(self.cdt)
-            x = random([size]).astype(self.cdt) +1j*x
+            x = random([size]).astype(self.cdt) + 1j*x
             y1 = ifft(fft(x))
             y2 = fft(ifft(x))
             self.assertTrue(y1.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
             self.assertTrue(y2.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
-            assert_array_almost_equal (y1, x)
-            assert_array_almost_equal (y2, x)
+            assert_array_almost_equal(y1, x)
+            assert_array_almost_equal(y2, x)
 
     def test_random_real(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
@@ -232,8 +257,8 @@ class _TestIFFTBase(TestCase):
                     "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
             self.assertTrue(y2.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
-            assert_array_almost_equal (y1, x)
-            assert_array_almost_equal (y2, x)
+            assert_array_almost_equal(y1, x)
+            assert_array_almost_equal(y2, x)
 
     def test_size_accuracy(self):
         # Sanity check for the accuracy for prime and non-prime sized inputs
@@ -260,16 +285,23 @@ class _TestIFFTBase(TestCase):
             self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, ifft, [])
+        assert_raises(ValueError, ifft, [[1,1],[2,2]], -5)
+
+
 class TestDoubleIFFT(_TestIFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
 
+
 class TestSingleIFFT(_TestIFFTBase):
     def setUp(self):
         self.cdt = np.complex64
         self.rdt = np.float32
 
+
 class _TestRFFTBase(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -298,16 +330,23 @@ class _TestRFFTBase(TestCase):
             y = fftpack.drfft(x)
             assert_array_almost_equal(y,y1)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, rfft, [])
+        assert_raises(ValueError, rfft, [[1,1],[2,2]], -5)
+
+
 class TestRFFTDouble(_TestRFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
 
+
 class TestRFFTSingle(_TestRFFTBase):
     def setUp(self):
         self.cdt = np.complex64
         self.rdt = np.float32
 
+
 class _TestIRFFTBase(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -315,7 +354,7 @@ class _TestIRFFTBase(TestCase):
     def test_definition(self):
         x1 = [1,2,3,4,1,2,3,4]
         x1_1 = [1,2+3j,4+1j,2+3j,4,2-3j,4-1j,2-3j]
-        x2= [1,2,3,4,1,2,3,4,5]
+        x2 = [1,2,3,4,1,2,3,4,5]
         x2_1 = [1,2+3j,4+1j,2+3j,4+5j,4-5j,2-3j,4-1j,2-3j]
 
         def _test(x, xr):
@@ -353,9 +392,9 @@ class _TestIRFFTBase(TestCase):
                     "Output dtype is %s, expected %s" % (y1.dtype, self.rdt))
             self.assertTrue(y2.dtype == self.rdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.rdt))
-            assert_array_almost_equal (y1, x, decimal=self.ndec,
+            assert_array_almost_equal(y1, x, decimal=self.ndec,
                                        err_msg="size=%d" % size)
-            assert_array_almost_equal (y2, x, decimal=self.ndec,
+            assert_array_almost_equal(y2, x, decimal=self.ndec,
                                        err_msg="size=%d" % size)
 
     def test_size_accuracy(self):
@@ -375,20 +414,28 @@ class _TestIRFFTBase(TestCase):
             self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, irfft, [])
+        assert_raises(ValueError, irfft, [[1,1],[2,2]], -5)
+
+
 # self.ndec is bogus; we should have a assert_array_approx_equal for number of
 # significant digits
+
 class TestIRFFTDouble(_TestIRFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
         self.ndec = 14
 
+
 class TestIRFFTSingle(_TestIRFFTBase):
     def setUp(self):
         self.cdt = np.complex64
         self.rdt = np.float32
         self.ndec = 5
 
+
 class Testfft2(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -399,9 +446,14 @@ class Testfft2(TestCase):
         # used
         x = numpy.ones((4,4,2))
         y = fft2(x, shape=(8,8), axes=(-3,-2))
-        y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3,  -2))
+        y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3, -2))
         assert_array_almost_equal(y, y_r)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, fft2, [[]])
+        assert_raises(ValueError, fft2, [[1,1],[2,2]], (4, -3))
+
+
 class TestFftnSingle(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -434,6 +486,7 @@ class TestFftnSingle(TestCase):
             self.assertTrue(y1.dtype == np.complex64)
             assert_array_almost_equal_nulp(y1, y2, 2000)
 
+
 class TestFftn(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -448,7 +501,7 @@ class TestFftn(TestCase):
         assert_array_almost_equal(fftn(x),direct_dftn(x))
 
     def test_axes_argument(self):
-        #plane == ji_plane, x== kji_space
+        # plane == ji_plane, x== kji_space
         plane1 = [[1,2,3],[4,5,6],[7,8,9]]
         plane2 = [[10,11,12],[13,14,15],[16,17,18]]
         plane3 = [[19,20,21],[22,23,24],[25,26,27]]
@@ -474,76 +527,76 @@ class TestFftn(TestCase):
         kij_space = [ij_plane1,ij_plane2,ij_plane3]
         x = array([plane1,plane2,plane3])
 
-        assert_array_almost_equal(fftn(x),fftn(x,axes=(-3,-2,-1))) # kji_space
+        assert_array_almost_equal(fftn(x),fftn(x,axes=(-3,-2,-1)))  # kji_space
         assert_array_almost_equal(fftn(x),fftn(x,axes=(0,1,2)))
-        y = fftn(x,axes=(2,1,0)) # ijk_space
+        y = fftn(x,axes=(2,1,0))  # ijk_space
         assert_array_almost_equal(swapaxes(y,-1,-3),fftn(ijk_space))
-        y = fftn(x,axes=(2,0,1)) # ikj_space
+        y = fftn(x,axes=(2,0,1))  # ikj_space
         assert_array_almost_equal(swapaxes(swapaxes(y,-1,-3),
-                                                   -1,-2)
-                                  ,fftn(ikj_space))
-        y = fftn(x,axes=(1,2,0)) # jik_space
+                                                   -1,-2),
+                                  fftn(ikj_space))
+        y = fftn(x,axes=(1,2,0))  # jik_space
         assert_array_almost_equal(swapaxes(swapaxes(y,-1,-3),
-                                                   -3,-2)
-                                  ,fftn(jik_space))
-        y = fftn(x,axes=(1,0,2)) # jki_space
+                                                   -3,-2),
+                                  fftn(jik_space))
+        y = fftn(x,axes=(1,0,2))  # jki_space
         assert_array_almost_equal(swapaxes(y,-2,-3),fftn(jki_space))
-        y = fftn(x,axes=(0,2,1)) # kij_space
+        y = fftn(x,axes=(0,2,1))  # kij_space
         assert_array_almost_equal(swapaxes(y,-2,-1),
                                   fftn(kij_space))
 
-        y = fftn(x,axes=(-2,-1)) # ji_plane
+        y = fftn(x,axes=(-2,-1))  # ji_plane
         assert_array_almost_equal(fftn(plane1),y[0])
         assert_array_almost_equal(fftn(plane2),y[1])
         assert_array_almost_equal(fftn(plane3),y[2])
-        y = fftn(x,axes=(1,2)) # ji_plane
+        y = fftn(x,axes=(1,2))  # ji_plane
         assert_array_almost_equal(fftn(plane1),y[0])
         assert_array_almost_equal(fftn(plane2),y[1])
         assert_array_almost_equal(fftn(plane3),y[2])
-        y = fftn(x,axes=(-3,-2)) # kj_plane
+        y = fftn(x,axes=(-3,-2))  # kj_plane
         assert_array_almost_equal(fftn(x[:,:,0]),y[:,:,0])
         assert_array_almost_equal(fftn(x[:,:,1]),y[:,:,1])
         assert_array_almost_equal(fftn(x[:,:,2]),y[:,:,2])
-        y = fftn(x,axes=(-3,-1)) # ki_plane
+        y = fftn(x,axes=(-3,-1))  # ki_plane
         assert_array_almost_equal(fftn(x[:,0,:]),y[:,0,:])
         assert_array_almost_equal(fftn(x[:,1,:]),y[:,1,:])
         assert_array_almost_equal(fftn(x[:,2,:]),y[:,2,:])
-        y = fftn(x,axes=(-1,-2)) # ij_plane
+        y = fftn(x,axes=(-1,-2))  # ij_plane
         assert_array_almost_equal(fftn(ij_plane1),swapaxes(y[0],-2,-1))
         assert_array_almost_equal(fftn(ij_plane2),swapaxes(y[1],-2,-1))
         assert_array_almost_equal(fftn(ij_plane3),swapaxes(y[2],-2,-1))
-        y = fftn(x,axes=(-1,-3)) # ik_plane
+        y = fftn(x,axes=(-1,-3))  # ik_plane
         assert_array_almost_equal(fftn(ik_plane1),swapaxes(y[:,0,:],-1,-2))
         assert_array_almost_equal(fftn(ik_plane2),swapaxes(y[:,1,:],-1,-2))
         assert_array_almost_equal(fftn(ik_plane3),swapaxes(y[:,2,:],-1,-2))
-        y = fftn(x,axes=(-2,-3)) # jk_plane
+        y = fftn(x,axes=(-2,-3))  # jk_plane
         assert_array_almost_equal(fftn(jk_plane1),swapaxes(y[:,:,0],-1,-2))
         assert_array_almost_equal(fftn(jk_plane2),swapaxes(y[:,:,1],-1,-2))
         assert_array_almost_equal(fftn(jk_plane3),swapaxes(y[:,:,2],-1,-2))
 
-        y = fftn(x,axes=(-1,)) # i_line
+        y = fftn(x,axes=(-1,))  # i_line
         for i in range(3):
             for j in range(3):
                 assert_array_almost_equal(fft(x[i,j,:]),y[i,j,:])
-        y = fftn(x,axes=(-2,)) # j_line
+        y = fftn(x,axes=(-2,))  # j_line
         for i in range(3):
             for j in range(3):
                 assert_array_almost_equal(fft(x[i,:,j]),y[i,:,j])
-        y = fftn(x,axes=(0,)) # k_line
+        y = fftn(x,axes=(0,))  # k_line
         for i in range(3):
             for j in range(3):
                 assert_array_almost_equal(fft(x[:,i,j]),y[:,i,j])
 
-        y = fftn(x,axes=()) # point
+        y = fftn(x,axes=())  # point
         assert_array_almost_equal(y,x)
 
     def test_shape_argument(self):
         small_x = [[1,2,3],[4,5,6]]
         large_x1 = [[1,2,3,0],[4,5,6,0],[0,0,0,0],[0,0,0,0]]
         y = fftn(small_x,shape=(4,4))
-        assert_array_almost_equal (y,fftn(large_x1))
+        assert_array_almost_equal(y,fftn(large_x1))
         y = fftn(small_x,shape=(3,4))
-        assert_array_almost_equal (y,fftn(large_x1[:-1]))
+        assert_array_almost_equal(y,fftn(large_x1[:-1]))
 
     def test_shape_axes_argument(self):
         small_x = [[1,2,3],[4,5,6],[7,8,9]]
@@ -552,16 +605,16 @@ class TestFftn(TestCase):
                                   [7,8,9,0],
                                   [0,0,0,0]])
         # Disable tests with shape and axes of different lengths
-        #y = fftn(small_x,shape=(4,4),axes=(-1,))
-        #for i in range(4):
+        # y = fftn(small_x,shape=(4,4),axes=(-1,))
+        # for i in range(4):
         #    assert_array_almost_equal (y[i],fft(large_x1[i]))
-        #y = fftn(small_x,shape=(4,4),axes=(-2,))
-        #for i in range(4):
+        # y = fftn(small_x,shape=(4,4),axes=(-2,))
+        # for i in range(4):
         #    assert_array_almost_equal (y[:,i],fft(large_x1[:,i]))
         y = fftn(small_x,shape=(4,4),axes=(-2,-1))
-        assert_array_almost_equal (y,fftn(large_x1))
+        assert_array_almost_equal(y,fftn(large_x1))
         y = fftn(small_x,shape=(4,4),axes=(-1,-2))
-        assert_array_almost_equal (y,swapaxes(\
+        assert_array_almost_equal(y,swapaxes(
             fftn(swapaxes(large_x1,-1,-2)),-1,-2))
 
     def test_shape_axes_argument2(self):
@@ -585,6 +638,10 @@ class TestFftn(TestCase):
         x = zeros((4, 4, 2))
         assert_raises(ValueError, fftn, x, shape=(8, 8, 2, 1))
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, fftn, [[]])
+        assert_raises(ValueError, fftn, [[1,1],[2,2]], (4, -3))
+
 
 class _TestIfftn(TestCase):
     dtype = None
@@ -609,16 +666,23 @@ class _TestIfftn(TestCase):
             assert_array_almost_equal_nulp(ifftn(fftn(x)),x,self.maxnlp)
             assert_array_almost_equal_nulp(fftn(ifftn(x)),x,self.maxnlp)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, ifftn, [[]])
+        assert_raises(ValueError, ifftn, [[1,1],[2,2]], (4, -3))
+
+
 class TestIfftnDouble(_TestIfftn):
     dtype = np.float64
     cdtype = np.complex128
     maxnlp = 2000
 
+
 class TestIfftnSingle(_TestIfftn):
     dtype = np.float32
     cdtype = np.complex64
     maxnlp = 3500
 
+
 class TestLongDoubleFailure(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -634,7 +698,7 @@ class TestLongDoubleFailure(TestCase):
         for f in [fft, ifft]:
             try:
                 f(x)
-                raise AssertionError("Type %r not supported but does not fail" % \
+                raise AssertionError("Type %r not supported but does not fail" %
                                      np.longcomplex)
             except ValueError:
                 pass
@@ -649,24 +713,26 @@ class TestLongDoubleFailure(TestCase):
         for f in [fft, ifft]:
             try:
                 f(x)
-                raise AssertionError("Type %r not supported but does not fail" % \
+                raise AssertionError("Type %r not supported but does not fail" %
                                      np.longcomplex)
             except ValueError:
                 pass
 
 
-
 class FakeArray(object):
     def __init__(self, data):
         self._data = data
         self.__array_interface__ = data.__array_interface__
 
+
 class FakeArray2(object):
     def __init__(self, data):
         self._data = data
+
     def __array__(self):
         return self._data
 
+
 class TestOverwrite(object):
     """
     Check input overwrite behavior of the FFT functions
@@ -684,9 +750,6 @@ class TestOverwrite(object):
                 routine.__name__, x.dtype, x.shape, fftsize, axis, overwrite_x)
             if not should_overwrite:
                 assert_equal(x2, x, err_msg="spurious overwrite in %s" % sig)
-            else:
-                if (x2 == x).all():
-                    raise AssertionError("no overwrite in %s" % sig)
 
     def _check_1d(self, routine, dtype, shape, axis, overwritable_dtypes):
         np.random.seed(1234)
